/*

  Electronic Wedding Wishing Well (EWWW)
      by Bill Porter       and     Mara Tuchman
        www.billporter.info   www.maratuchman.info
        <3<3<3<3<3  www.billandmara.com  <3<3<3<3<3<3
        
      TLC5940 Library by Alex Leone
      Wtv020 library by Diego J. Arevalo
      
      Library notes:
      
    Basic Pin setup:
    ------------                                  ---u----
    ARDUINO   13|-> SCLK (pin 25)           OUT1 |1     28| OUT channel 0
              12|                           OUT2 |2     27|-> GND (VPRG)
              11|-> SIN (pin 26)            OUT3 |3     26|-> SIN (pin 11)
              10|-> BLANK (pin 23)          OUT4 |4     25|-> SCLK (pin 13)
               9|-> XLAT (pin 24)             .  |5     24|-> XLAT (pin 9)
               8|                             .  |6     23|-> BLANK (pin 10)
               7|                             .  |7     22|-> GND
               6|                             .  |8     21|-> VCC (+5V)
               5|                             .  |9     20|-> 2K Resistor -> GND
               4|                             .  |10    19|-> +5V (DCPRG)
               3|-> GSCLK (pin 18)            .  |11    18|-> GSCLK (pin 3)
               2|                             .  |12    17|-> SOUT
               1|                             .  |13    16|-> XERR
               0|                           OUT14|14    15| OUT channel 15
    ------------                                  --------


    -  The 2K resistor between TLC pin 20 and GND will let ~20mA through each
       LED.  To be precise, it's I = 39.06 / R (in ohms).  This doesn't depend
       on the LED driving voltage.

    This library uses the PWM output ability of digital pins 3, 9, 10, and 11.
    Do not use analogWrite(...) on these pins.
    
    
    
    My configuration:
    
    
    Digital Pins
    0  
    1  
    2  Sensor
    3  TLC GSCLK
    4  MUSIC Reset
    5  MUSIC CLK
    6  MUSIC data
    7  MUSIC busy
    8  
    9   TLC XLAT
    10  TLC BLANK
    11  TLC SIN
    12
    13  TLC SCLK
    
    LED channels
    0  RGBLED1R
    1  RGBLED1G
    2  RGBLED1B
    3  RGBLED2R
    4  RGBLED2G
    5  RGBLED2B
    6-10  WHITE LEDS
    11-12 YELLOW LEDS
    13-15 BLUE LEDS
    
 */

#include "Tlc5940.h"
#include "tlc_fades.h"
#include <Wtv020sd16p.h>
#include "VixenShow.cpp"

/*
Create an instance of the Wtv020sd16p class.
 1st parameter: Reset pin number.
 2nd parameter: Clock pin number.
 3rd parameter: Data pin number.
 4th parameter: Busy pin number.
 */
Wtv020sd16p wtv020sd16p(4,5,6,7);



void setup()
{
  /* Call Tlc.init() to setup the tlc.
     You can optionally pass an initial PWM value (0 - 4095) for all channels.*/
  Tlc.init();
  
  Serial.begin(115200);
  //Initializes the music module.
  wtv020sd16p.reset();
  
  randomSeed(analogRead(0));
  
  pinMode(2, INPUT);
}

void loop(){
  
  //TODO:create a gentle light show while idle
  //when sensor triggered, trigger sound effect and wild light show
  if(digitalRead(2) == LOW)
    playShow();
  else
    idleShow();
   //delay(30000);
  
}

void playShow(){
  
  //start music track
  wtv020sd16p.asyncPlayVoice(0);
  delay(100);
  
  int temp = 0;
  long start = 0;
  
  //start playing back show file and run LEDS
  for(int i = 0; i<vixen_frames; i++){
    start = millis();
    for(int y = 0; y<vixen_channels; y++){
      temp = pgm_read_byte(&(vixen_show_data[y][i]));
      if(y<6)
        temp = map(temp,0,255,0,4095); //scale brighter for RGB LEDs      
      else
        temp = map(temp,0,255,0,2048); //scale to lower half of TLC range.
      Tlc.set(y, temp);
    }
    Tlc.update();
    delay(vixen_frameduration - (millis() - start));
  }
  
}

void idleShow(){
  //for gentle fades, use the fade scheduling functions of the TLC library.
  //use:  tlc_addFade(channel, StartValue, EndValue, startMillis, endMillis);
  uint32_t new_value, duration, startMillis, endMillis;
  //run a loop to check fades on single color leds and assign one if not. 
  
  for(int i= 6; i<=10; i++){
    if(!tlc_isFading(i)){
      //assign new fade
      Serial.print("new fade for: ");
      Serial.println(i);
      new_value = random(0,800);
      duration = random(300,1500);
      startMillis = millis() + 50;
      endMillis = startMillis + duration;
      tlc_addFade(i, Tlc.get(i), new_value, startMillis, endMillis);
    } 
  }
  tlc_updateFades();
}

